package net.jxta.chat.core;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.TimerTask;

import javax.swing.DefaultListModel;

import net.jxta.discovery.DiscoveryEvent;
import net.jxta.discovery.DiscoveryListener;
import net.jxta.discovery.DiscoveryService;
import net.jxta.protocol.DiscoveryResponseMsg;
import net.jxta.protocol.PeerGroupAdvertisement;

/**
 * @author Matteo Tanca [tanca@cli.di.unipi.it]
 * @version 0.1
 * Thread che si occupa di effettuare ricerche periodiche nel gruppo principale per
 * individuare nuove chat
 */

public class ChatFinder extends TimerTask implements DiscoveryListener {
	
	private Hashtable<String, PeerGroupAdvertisement> table;
	private DiscoveryService discoverySv;
	private DefaultListModel listModel = null;
	
	protected ChatFinder (DiscoveryService d) {
		table = new Hashtable<String, PeerGroupAdvertisement>();
		discoverySv = d;
        discoverySv.addDiscoveryListener(this);
	}
	
	/**
	 * Metodo invocato periodicamente (in quanto ChatFinder estende TimerTask), che
	 * consente l'aggiornamento della tabella delle chat conosciute
	 */
	public void run() {

		try {
          System.out.println("Invio di un messaggio di discovery alla ricerca di" +
                		" nuove chat");
          /* Ricerca in remoto di PeerGroup, effettuata su tutti i peer, senza
           * restrizioni sul tipo di chat trovate e con limite di 50 risultati
           */
          discoverySv.getRemoteAdvertisements(null, DiscoveryService.GROUP,null, null, 50);
        } catch(Exception e) {
        	System.out.println("Problemi nell'esecuzione del ChatFinder (metodo run())");
            e.printStackTrace();
        }
    }
	
    /**
     * Implementazione del metodo discoveryEvent, richiesta per poter implementare l'interfaccia
     * DiscoveryListener, che permette di ricevere i risultati delle ricerche in maniera
     * asincrona. Si noti che è permessa l'esistenza di chat aventi lo stesso nome, in quanto
     * la chiave utilizzata e' data dalla concatenazione di nome e GID della chat
     */
    public void discoveryEvent (DiscoveryEvent ev) {

        DiscoveryResponseMsg res = ev.getResponse();

        PeerGroupAdvertisement chatAdv = null;
        Enumeration enumeration = res.getAdvertisements();
        
        /* Trovata qualche chat */
        if (enumeration != null ) {
            while (enumeration.hasMoreElements()) {
                chatAdv = (PeerGroupAdvertisement) enumeration.nextElement();
                addChat(chatAdv);
            }
        }
    }
    
    /**
     * Metodo per l'inserzione di un advertisement di una chat. Utilizzato anche alla
     * creazione di una chat per far risultare immediatamente la sua presenza in lista
     * @param newChatAdv L'advertisement da inserire
     */
    public void addChat (PeerGroupAdvertisement newChatAdv) {
    	String key = newChatAdv.getName() + " [" + newChatAdv.getDescription() + "] (" + newChatAdv.getPeerGroupID() + ")";
    	
    	/* Inserisce l'elemento e nel caso non sia già presente aggiorna il modello */
    	if (table.put(key, newChatAdv) == null)
    		addToListModel(key);
    }
    
    /**
     * Metodo che restituisce la tabella delle chat, aggiornata all'ultima ricerca effettuata
     * dal ChatFinder (a meno di inserzioni manuali effettuate dal PeerCore)
     * @return Hastable La tabella delle chat finora trovate dal ChatFinder
     */
    public Hashtable<String, PeerGroupAdvertisement> getChatTable () {
    	return table;
    }
    
    /**
     * Metodo per impostare il modello di lista da aggiornare con i risultati delle ricerche
     * @param lm DafaultListModel
     */
    public void setListModel (DefaultListModel lm) {
    	listModel = lm;
    }
    
    
    /**
     * Metodo privato per il refresh del modello usato per aggiornare la lista delle chat
     * visualizzata dall'interfaccia
     * @return Boolean flag di corretto inserimento
     */
    private boolean addToListModel (String key) {
    	if (listModel == null)
    		return false;
    	try {
    		listModel.addElement(key);
    	} catch (NullPointerException npe) {
    		//in caso di chiave nulla non inserire
    		return false;
    	}
    	return true;
    }
}
