package net.jxta.chat.core;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Timer;

import javax.swing.DefaultListModel;

import net.jxta.chat.interfaces.ChatInterface;
import net.jxta.chat.presence.StatusService;
import net.jxta.chat.presence.StatusServiceUnavaliableException;
import net.jxta.discovery.DiscoveryService;
import net.jxta.document.AdvertisementFactory;
import net.jxta.endpoint.Message;
import net.jxta.endpoint.MessageElement;
import net.jxta.endpoint.StringMessageElement;
import net.jxta.exception.ServiceNotFoundException;
import net.jxta.id.IDFactory;
import net.jxta.peergroup.PeerGroup;
import net.jxta.pipe.InputPipe;
import net.jxta.pipe.OutputPipe;
import net.jxta.pipe.PipeID;
import net.jxta.pipe.PipeMsgEvent;
import net.jxta.pipe.PipeMsgListener;
import net.jxta.pipe.PipeService;
import net.jxta.platform.ModuleClassID;
import net.jxta.protocol.PipeAdvertisement;

/**
 * @author Matteo Tanca [tanca@cli.di.unipi.it]
 * @version 0.1
 * Classe che si occupa di gestire la singola chat
 *
 */
public class ChatHandler implements PipeMsgListener {
	
	/* Intervallo di refresh per la ricerca di peer e il refresh dello stato degli utenti (default 10")*/
	private final long REFRESH_PERIOD = 1000 * 10;
	private final String SENDER_TAG = "msgSender";
	private final String MESSAGE_TEXT_TAG = "msgText";
	
	/* 
	 * Seed prestabilito per la creazione del pipeID, mediante combinazione
	 * con il PeerGroupID della chat
	 */
	private static final byte[]  PROPAGATE_PIPE_ID_SEED = 
    { (byte)0xD1, (byte)0xD1, (byte)0xD1, (byte)0xD1, 
      (byte)0xD1, (byte)0xD1, (byte)0xD1, (byte)0xD1,
      (byte)0xD1, (byte)0xD1, (byte)0xD1, (byte)0xD1, 
      (byte)0xD1, (byte)0xD1, (byte)0xD1, (byte)0xD1 };
	
	private PeerCore peercore;
	private DiscoveryService discoverySv;
	private PipeService pipeSv;
	private StatusService usersInfoSv;
	private PeerGroup chat;
	private OutputPipe propagatePipe = null;
	private InputPipe inPipe = null;
	private PipeAdvertisement propagatePipeAdv = null;
	private PeersAndStatusFinder pAndSFinder = null;
	private ChatInterface cInterface;
	private int status = StatusService.ONLINE_STATUS;
	private String sender = null;
	private DefaultListModel listModel;
	private boolean hasToLogChanges = true;
	
	
	public ChatHandler (PeerCore pc, PeerGroup p, ChatInterface cI, DefaultListModel pLM, String nick) {
		peercore = pc;
		chat = p;
		cInterface = cI;
		discoverySv = p.getDiscoveryService();
		pipeSv = p.getPipeService();
		initializeUsersInfoSv();
		listModel = pLM;
		sender = nick;
		pAndSFinder = new PeersAndStatusFinder(discoverySv, usersInfoSv, this, pLM);
		propagatePipeAdv = (PipeAdvertisement)AdvertisementFactory.newAdvertisement(
				PipeAdvertisement.getAdvertisementType());
		propagatePipeAdv.setPipeID((PipeID)IDFactory.newPipeID(chat.getPeerGroupID(), PROPAGATE_PIPE_ID_SEED));
		propagatePipeAdv.setType(PipeService.PropagateType);
		try {	
			inPipe = pipeSv.createInputPipe(propagatePipeAdv, this);
		} catch (IOException ioe) {
			System.out.println("Problemi nella creazione e registrazione della pipe di input");
			ioe.printStackTrace();
		}
		try {
			propagatePipe = pipeSv.createOutputPipe(propagatePipeAdv, 0);
		} catch (IOException ioe) {
			System.out.println("Problemi nella creazione e registrazione della pipe di output");
			ioe.printStackTrace();
		}
		Timer pAndSTimer = new Timer();
		pAndSTimer.scheduleAtFixedRate(pAndSFinder, 0, REFRESH_PERIOD);
	}
	
	/**
	 * Implementazione richiesta dall'interfaccia PipeMsgListener
	 * @param msgEvent PipeMsgEvent L'evento contenitore del messaggio
	 */
	public void pipeMsgEvent (PipeMsgEvent msgEvent) {
		if (status == StatusService.OFFLINE_STATUS)
			return;
		Message msg = null;
		
		String msgSender = null;
		String msgText = null;
		try {
			msg = msgEvent.getMessage();
			MessageElement senderElement = msg.getMessageElement(SENDER_TAG);
			msgSender = senderElement.toString();
			MessageElement textElement = msg.getMessageElement(MESSAGE_TEXT_TAG);
			msgText = textElement.toString();
		} catch (Exception e) {
			System.out.println("Problemi nella ricezione del messaggio");
			e.printStackTrace();
			return;
		}
		
		if (msgSender != null && msgText != null) {
			Calendar cal = new GregorianCalendar();
		    int hour = cal.get(Calendar.HOUR_OF_DAY);
		    int min = cal.get(Calendar.MINUTE);
		    int sec = cal.get(Calendar.SECOND); 
			String s = "{" + hour + ":" + min + ":" + sec + "} " +
					   msgSender + ChatInterface.SEPARATOR + msgText;
			cInterface.appendToHistory(s);
		}
	}

	/**
	 * Metodo che si occupa di costruire il messaggio contenente mittente (nick
	 * utilizzato nella chat) e stringa ricevuta come argomento ed inviarlo
	 * sulla pipe di propagazione
	 * @param msg Stringa contenente il messaggio di cui e' stato richiesto l'invio
	 */
	public void sendMsg(String msg) {
		if (status == StatusService.OFFLINE_STATUS)
			return;
		Message m = new Message();
		StringMessageElement senderMsgEl = new StringMessageElement(SENDER_TAG, sender, null);
		m.addMessageElement(null, senderMsgEl);
		StringMessageElement msgTextEl = new StringMessageElement(MESSAGE_TEXT_TAG, msg, null);
		m.addMessageElement(msgTextEl);
		try {
			propagatePipe.send(m);
		} catch (IOException ioe) {
			System.out.println("Impossibile spedire il messaggio");
			cInterface.appendToHistory("Errore nell'invio del messaggio <" + msg + ">\n");
		}
	}
	
	/**
	 * Metodo che permette di cambiare il nickname di chat
	 * @param nick Stringa identificativa
	 */
	public void setNickname (String nick) {
		sender = nick;
		pAndSFinder.refreshLocalInfo();
		cInterface.setTitle(sender + " @ " + chat.getPeerGroupName() + " [" + chat.getPeerGroupAdvertisement().getDescription() + "] (" + "GID: " + chat.getPeerGroupID() + ")");
	}
	
	/**
	 * Metodo che restituisce la variabile d'istanza pAndPFinder
	 * @return PeersAndPipesFinder Il TimerTask responsabile della ricerca periodica
	 * dello stato dei peer in chat e delle loro pipe per la messaggistica istantanea
	 */
	public PeersAndStatusFinder getPeersAndStatusFinder () {
		return pAndSFinder;
	}
	
	private void initializeUsersInfoSv () throws StatusServiceUnavaliableException {
		try {
			ModuleClassID mcID = (ModuleClassID)IDFactory.fromURI(new URI(StatusService.refModuleClassID));
			usersInfoSv = (StatusService)chat.lookupService(mcID);
		} catch (URISyntaxException use) {
			throw new StatusServiceUnavaliableException();
		} catch (ServiceNotFoundException snfe) {
			throw new StatusServiceUnavaliableException("Servizio informativo " +
					"non disponibile (ServiceNotFoundException)");
		}
		
	}

	/**
	 * Metodo che permette di ottenere l'attuale nickname di chat
	 * @return nick Stringa identificativa
	 */
	public String getNickname () {
		return sender;
	}
	
	/**
	 * Metodo che permette di ottenere l'attuale stato di chat
	 * @return Stringa indicativa dello stato di presenza
	 */
	public String getPresenceInfo() {
		switch (status) {
			case StatusService.ONLINE_STATUS:
				return "In linea";
			case StatusService.AWAY_STATUS:
				return "Non al computer";
			case StatusService.BUSY_STATUS:
				return "Occupato";
			case StatusService.BACK_SOON_STATUS:
				return "Torno subito";
			case StatusService.OFFLINE_STATUS:
				return "Non in linea";
		}
		return null;
	}
	
	public int getPresenceIndex () {
		return status;
	}
	
	public String getLocalPeerID () {
		return chat.getPeerID().toString();
	}
	
	/**
	 * Metodo per l'impostazione dello stato utente (presence information)
	 * @param presenceIndex Intero indicativo dello stato utente, come
	 * 		  indicato nello StatusService
	 */
	public void setPresenceInfo (int presenceIndex) {
		if(status == presenceIndex)
			return;
		if (status == StatusService.OFFLINE_STATUS) {
			listModel.clear();
			pAndSFinder = new PeersAndStatusFinder(discoverySv, usersInfoSv, this, listModel);
			Timer pAndSTimer = new Timer();
			pAndSTimer.scheduleAtFixedRate(pAndSFinder, 0, REFRESH_PERIOD);
		}
		status = presenceIndex;
		if (presenceIndex == StatusService.OFFLINE_STATUS) {
			usersInfoSv.publishUserInfo(status, sender);
			usersInfoSv.removeListener(pAndSFinder);
			discoverySv.removeDiscoveryListener(pAndSFinder);
			listModel.clear();
			listModel.addElement(sender + " (Non in linea)");
			pAndSFinder.cancel();
			for (int i = 0; i < 4; i ++) {
				try {
					Thread.sleep(3000);
				} catch (InterruptedException ie) {
					
				}
				usersInfoSv.publishUserInfo(status, sender);
			}
		}
		else
			pAndSFinder.refreshLocalInfo();
	}
	
	/**
	 * Metodo per il log dei cambiamenti di nick degli utenti nella
	 * cronologia
	 * @param oldNick Il vecchio nickname dell'utente
	 * @param newNick Il nuovo nickname dell'utente
	 */
	public void logNickChange (String oldNick, String newNick) {
		if (hasToLogChanges && oldNick != null)
			cInterface.appendToHistory("--\n" + oldNick + " e' adesso noto come " + newNick + "\n--\n");
	}
	
	/**
	 * Metodo per il log dei cambiamenti di stato degli utenti nella
	 * cronologia
	 * @param nick Nickname dell'utente che ha cambiato stato
	 * @param status Nuovo stato dell'utente
	 */
	public void logStatusChange (String nick, int status) {
		if (hasToLogChanges) {
			String statusString = null;
	    	switch (status) {
	    		case StatusService.OFFLINE_STATUS:
	    			statusString = "Non in linea";
	    			break;
	    		case StatusService.ONLINE_STATUS:
	    			statusString = "In linea";
	    			break;
	    		case StatusService.AWAY_STATUS:
	    			statusString = "Non al computer";
	    			break;
	    		case StatusService.BUSY_STATUS:
	    			statusString = "Occupato";
	    			break;
	    		case StatusService.BACK_SOON_STATUS:
	    			statusString = "Torno subito";
	    	}
			cInterface.appendToHistory("--\n" + nick + " ha impostato il suo stato su: " + statusString + "\n--\n");
		}
	}
	
	/**
	 * Semplice metodo per set e unset del flag che regola la registrazione
	 * in cronologia dei cambiamenti di nick e stato degli utenti
	 */
	public void invertLogFlag () {
		hasToLogChanges = !hasToLogChanges;
	}
	
	/**
	 * Metodo invocato dal listener di eventi dell'interfaccia di chat
	 * al momento della chiusura (tramite 'Esci' o tramite chiusura).
	 * Si occupa di terminare l'esecuzione dei TimerTask attivi e deregistrare
	 * i listener, impostando inoltre lo stato dell'utente come Non in linea
	 */
	public void dispose () {
		status = StatusService.OFFLINE_STATUS;
		usersInfoSv.publishUserInfo(status, sender);
		usersInfoSv.removeListener(pAndSFinder);
		discoverySv.removeDiscoveryListener(pAndSFinder);
		pAndSFinder.cancel();
		/* Rimozione dall'elenco delle chat attive */
		String key = chat.getPeerGroupAdvertisement().getName() + " [" + chat.getPeerGroupAdvertisement().getDescription() + "] (" + chat.getPeerGroupAdvertisement().getPeerGroupID() + ")";
		peercore.chatLeaved(key);
		cInterface.dispose();
		for (int i = 0; i < 4; i ++) {
			try {
				Thread.sleep(3000);
			} catch (InterruptedException ie) {
				
			}
			usersInfoSv.publishUserInfo(status, sender);
		}
	}
}
