package net.jxta.chat.core;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Timer;
import java.util.Vector;

import net.jxta.chat.interfaces.ChatInterface;
import net.jxta.chat.presence.AdvertisementCreationUtilities;
import net.jxta.chat.presence.StatusService;
import net.jxta.chat.presence.StatusServiceImpl;
import net.jxta.discovery.DiscoveryService;
import net.jxta.document.MimeMediaType;
import net.jxta.document.StructuredDocument;
import net.jxta.exception.PeerGroupException;
import net.jxta.id.IDFactory;
import net.jxta.impl.peergroup.StdPeerGroupParamAdv;
import net.jxta.peergroup.PeerGroup;
import net.jxta.peergroup.PeerGroupFactory;
import net.jxta.peergroup.PeerGroupID;
import net.jxta.platform.ModuleSpecID;
import net.jxta.platform.NetworkConfigurator;
import net.jxta.protocol.ModuleClassAdvertisement;
import net.jxta.protocol.ModuleImplAdvertisement;
import net.jxta.protocol.ModuleSpecAdvertisement;
import net.jxta.protocol.PeerGroupAdvertisement;
import net.jxta.rendezvous.RendezVousService;

/**
 * @author Matteo Tanca [tanca@cli.di.unipi.it]
 * @version 0.1
 * Nucleo dell'applicazione di chat. Si occupa di effettuare l'inizializzazione standard
 * comune a tutte le applicazioni JXTA, di effettuare l'accesso al PeerGroup di default per
 * l'applicazione di chat (eventualmente creandolo qualora non sia disponibile), di creare le
 * stanze di chat e effettuare le operazioni per parteciparvi
 */

public class PeerCore {
	
	/* Serie di informazioni costanti, relative all'applicazione ed al suo gruppo principale */
	public final static String APP_VERSION = "0.1";
	public final static String APP_NAME = "JXTA_chat";
	public final static String MAINGROUP_NAME = "MainChatGroup";
	public final static String MAINGROUP_ID = "urn:jxta:uuid-4d6172676572696e204272756e6f202002";
	private static final String refUsersStatusServiceModPeerGroupSpec = 
        "urn:jxta:uuid-DEADBEEFDEAFBABAFEEDBABE00000001CB18295"
        + "F0DE94F99983AA2F00C1DE42F06";
	
	private PeerGroup netPG = null;    /* Il gruppo JXTA di default */
	private PeerGroup mainGroup = null; /* Il gruppo di default dell'applicazione */
	
	private ChatFinder chatFinder = null; /* TimerTask di ricerca delle chat */
	
	/* Attesa prevista per il tentativo di ricerca del gruppo principale dell'applicazione */
	private final int mainTimeout = 30000;
	/* Periodo di refresh della lista delle chat (30 s) */
	private final long CHAT_REFRESH_PERIOD = 30000;
	/* Periodo di attesa (in sec) di una connessione ad un Rendezvous prima dell'auto elezione in fase di avvio */
	private final int RDV_PROMOTION_TIMEOUT = 200;
	/* ModuleImplAdvertisement dell'implementazione del gruppo di chat */
	ModuleImplAdvertisement impAdv = null;
	
	/* Servizio di discovery, necessario per la ricerca e pubblicazione degli advertisement */
    private DiscoveryService discoverySv = null;
    
    /*
     * Vettore di controllo, che impedisce di effettuare join multiple
     * alla stessa stanza di chat
     */
    private Vector<String> joinedChats = null;
    
    /**
     * Metodo utilizzato alla chiusura (dell' interfaccia) di una
     * stanza di chat, per segnalare al PeerCore che non si sta
     * piu' partecipando alla chat, ripristinando la possibilita'
     * di accedervi
     * @param key Chiave dell'elemento da rimuovere
     */
    public void chatLeaved (String key) {
    	joinedChats.remove(key);
    }
    
    /**
     * Metodo di controllo sul vettore delle chat a cui si sta gia'
     * partecipando, per evitare partecipazioni multiple alla medesima chat
     * @param key
     * @return true se si sta gia' partecipando, false altrimenti
     */
    public boolean isInChat (String key) {
    	if (joinedChats.contains(key))
    		return true;
    	else 
    		return false;
    }
    /**
     * Metodo che ricerca nella tabella hash proveniente dal chatFinder il PeerGroupAdvertisement
     * corrispondente alla stringa passata come parametro. Qualora esista tenta di effettuare la
     * join, mediante invocazione del metodo appropriato
     * @param key La chiave di ricerca nella Hashtable
     */
    public void joinChat (String key, String nickName) {
    	
    	Hashtable<String, PeerGroupAdvertisement> table = chatFinder.getChatTable();
    	PeerGroupAdvertisement chatAdvertisement = table.get(key);
    	if (chatAdvertisement != null) /* Trovata un'associazione */
    		if (joinChat(chatAdvertisement, nickName))
    	    	joinedChats.add(key);
    }
     
    /**
     * Metodo privato per l'inizializzazione dell'applicazione. Invoca una serie di metodi
     * privati per accedere al WorldPeerGroup, partecipare (eventualmente creandolo qualora
     * non sia raggiungibile) al gruppo di default per l'applicazione di chat. Inizializza
     * anche il TimerTask per l'aggiornamento dell'elenco delle chat preesistenti
     * @return boolean Flag di corretta inizializzazione
     */
    public boolean initialize () {
    	System.out.println("Inizializzazione di " + APP_NAME + " " + APP_VERSION);
    	
    	/* Configurazione opzioni di rete JXTA */
    	 NetworkConfigurator config = new NetworkConfigurator();
    	 if (!config.exists()) {
    		 int r = (int)(Math.random() * 1000);
    		 config.setName("jxta_config");
    		 config.setPrincipal("node" + r);
    		 config.setPassword("pass");
    		 config.setMulticastAddress("224.0.0.1");
    		 config.setMulticastPort(1232);
    		 try {
    			 config.save();
    		 } catch (IOException io) {
    			 System.out.println("Errore nel salvataggio della configurazione");
    			 io.printStackTrace();
    		 }
    	 } 
    	 else {
    		 File pc = new File(config.getHome(), "PlatformConfig");
    		 try {
    			 config.load(pc.toURI());
    			 config.setMulticastAddress("224.0.0.1");
    			 config.setMulticastPort(1232);
    		 } catch (Exception io) {
    			 System.out.println("Errore nel caricamento della configurazione");
    			 io.printStackTrace();
    		 }
    	 }
            
    	if(!startJxta()) { /* inizializzazione standard per le applicazioni JXTA */
    		System.out.println("Inizializzazione JXTA WorldPeerGroup fallita");
			return false;
    	}

		System.out.println("Inizializzazione JXTA NetPeerGroup completata");
    	
    	/*
    	 * Inizializzazione del gruppo di default per JXTA_chat
    	 */
    	if (!initMainGroup()) {
    		System.out.println("Impossibile completare l'inizializzazione del " + MAINGROUP_NAME);
    		return false;
    	}
    	
    	System.out.println("Inizializzazione " + MAINGROUP_NAME + " completata");
    	
    	try {
            impAdv = mainGroup.getAllPurposePeerGroupImplAdvertisement();
            /* Creazione degli advertisement necessari alla pubblicazione dello UsersStatusService */
            ModuleClassAdvertisement usrsStatusServiceClassAdv = 
         	   AdvertisementCreationUtilities.createModuleClassAdvertisement(StatusService.refModuleClassID,
         			   StatusService.SERVICE_NAME, StatusService.SERVICE_DESCRIPTION);
            ModuleSpecAdvertisement usrsStatusServiceSpecAdv =
         	   AdvertisementCreationUtilities.createModuleSpecAdvertisement(StatusServiceImpl.refModuleSpecID,
         			   StatusServiceImpl.SERVICE_NAME, StatusServiceImpl.SPEC_DESCRIPTION);
            ModuleImplAdvertisement usrsStatusServiceImplAdv =
         	   AdvertisementCreationUtilities.createModuleImplAdvertisement(impAdv, usrsStatusServiceSpecAdv,
         			   							StatusServiceImpl.SERVICE_DESCRIPTION,
         			   							"net.jxta.chat.presence.StatusServiceImpl");
            
            
            /* Creati gli advertisement, si può modificare l'advertisement di default del Peergroup */
            StdPeerGroupParamAdv peerGroupParameters = new StdPeerGroupParamAdv(impAdv.getParam());
            peerGroupParameters.getServices().put(usrsStatusServiceClassAdv.getModuleClassID(), usrsStatusServiceImplAdv);
            /* Eliminazione delle applicazioni (JXTA shell) */
            peerGroupParameters.setApps(new Hashtable());
            /* Dopo aver modificato la mappa dei servizi, reimposta il parametro dell'advertisement */
            impAdv.setParam((StructuredDocument)peerGroupParameters.getDocument(MimeMediaType.XMLUTF8));
            
            /* Modifica del ModuleSpecID, per rendere effettivo il cambio di servizi */
            impAdv.setModuleSpecID((ModuleSpecID)IDFactory.fromURI(new URI(refUsersStatusServiceModPeerGroupSpec)));
            
            /* Terminate le modifiche, si pubblicano gli Advertisement */
            discoverySv.publish(usrsStatusServiceClassAdv);
            discoverySv.remotePublish(usrsStatusServiceClassAdv);
            discoverySv.publish(impAdv);
            discoverySv.remotePublish(impAdv);
         } catch (Exception e) {
             System.out.println("Fallimento nell'inizializzazione degli " +
             		"advertisement dell'implementazione delle chatroom");
             e.printStackTrace();
         }
         
    	Timer chatSearchTimer = new Timer();
    	chatFinder = new ChatFinder(discoverySv);
    	chatSearchTimer.scheduleAtFixedRate(chatFinder, 0, CHAT_REFRESH_PERIOD);
    	joinedChats = new Vector<String>();
    	System.out.println("Inizializzazione completata\nE' ora possibile creare stanze di chat e parteciparvi");
    	return true;
    }
    
    /**
     * Metodo che restituisce il riferimento al TimerTask chatFinder
     * @return ChatFinder
     */
    public ChatFinder getChatFinder() {
    	return chatFinder;
    }
    
    /**
     * Metodo privato tipicamente presente in tutte le applicazioni JXTA. Invoca
     * il metodo predefinito nella libreria JXTA per la creazione/join del NetPeerGroup
     * (il gruppo di default per ogni applicazione JXTA, detto anche WorldPeerGroup).
     * Ottiene anche il servizio di discovery del gruppo di default, per poter successivamente
     * pubblicare in remoto la presenza del gruppo principale per JXTA_chat
     * @return boolean Flag sulla corretta terminazione del metodo
     */
    private boolean startJxta() {
        try {
            netPG = PeerGroupFactory.newNetPeerGroup();
        } catch (PeerGroupException pge) {
            System.out.println("Errore nella creazione del gruppo di default");
            return false;
        }
        
        discoverySv = netPG.getDiscoveryService();
        return true;
    }
    
    /**
     * Metodo privato per la la ricerca e partecipazione al gruppo principale dell'applicazione
     * (mainChatGroup), all'interno del quale sono create tutte le chat. Si occupa della ricerca
     * del gruppo, creandolo solo qualora non sia in grado di trovarlo (ossia qualora non
     * esista o non sia apparentemente raggiungibile). Una volta volta trovato (o creato) il
     * gruppo effettua la join su di esso ed associa la variabile discoverySv al servizio
     * di discovery del mainChatGroup
     * @return boolean Flag sulla corretta inizializzazione del gruppo principale
     */
    private boolean initMainGroup() {
        
    	System.out.println("Inizializzazione del gruppo principale per l'applicazione");
    	
    	/* Advertisement del mainChatGroup */
    	PeerGroupAdvertisement mCGAdv = searchForMainGroup();
    	
    	if (mCGAdv == null)	 /* E' necessario creare il mainChatGroup ex-novo */
    		mainGroup = createMainGroup();
		else
			try {
				mainGroup = netPG.newGroup(mCGAdv);
			} catch (PeerGroupException pge) {
				System.out.println("Problema nella creazione del " + MAINGROUP_NAME + " a partire " +
						"dall'advertisement trovato dalla ricerca. Potrebbe essere necessario " +
						"eliminare la cache locale");
				pge.printStackTrace();
				return false;
			}
        
        /* Associa il nuovo servizio di discovery, per permettere la ricerca delle chat esistenti */
        discoverySv = mainGroup.getDiscoveryService();
        
        return true;	/* L'inizializzazione e' andata a buon fine */
    }
    
    /**
     * Metodo privato per la ricerca del gruppo principale dell'applicazione
	 * @return PeerGroupAdvertisement L'advertisement del mainChatGroup, se trovato, altrimenti
	 * null
	 */
	private PeerGroupAdvertisement searchForMainGroup() {
		try {
			waitRDVOrPromote(netPG, RDV_PROMOTION_TIMEOUT);
		} catch (Exception e) {e.printStackTrace();}
		
		Enumeration<PeerGroupAdvertisement> enumeration = null;
		boolean found = false;	/* flag di avvenuto ritrovamento di uno o più advertisement */
		int retries = 3;	/* 3 tentativi di ricerca */
		 
		while (!found && retries > 0) {
			retries--;
			System.out.println("Ricerca del gruppo principale in corso " +
					"(tentativi rimasti: " + retries + ")");
			try {
				/* ricerca mediante PeerGroup ID */
				enumeration = discoverySv.getLocalAdvertisements(DiscoveryService.GROUP, "GID", MAINGROUP_ID);
				if (enumeration != null && enumeration.hasMoreElements()) {
					found = true;
					System.out.println("Trovato advertisement del gruppo principale");
					break;
				}
				
				discoverySv.getRemoteAdvertisements(null, DiscoveryService.GROUP, "GID", MAINGROUP_ID, 1);
				try {
					Thread.sleep(mainTimeout);
				} catch (InterruptedException ie) {
					ie.printStackTrace();
				}
			} catch (IOException ioe) {
				System.out.println("Problema nella ricerca dell'advertisement relativo " +
						"al " + MAINGROUP_NAME);
				ioe.printStackTrace();
				return null;
			}
		}
		
		/*
		 * Se e' stato trovato qualcosa viene restituito il primo risultato disponibile
		 * dell'enumerazione, altrimenti null, ad indicare il fallimento della ricerca
		 */
		if (!found)
			return null;
		else
			return (PeerGroupAdvertisement) enumeration.nextElement();
	}

	/**
     * Metodo privato per la creazione e pubblicazione in remoto del gruppo principale della
     * applicazione
     * @return PeerGroup mainChatGroup Il gruppo principale, creato e pubblicato dal metodo
     */
    private PeerGroup createMainGroup () {
    	
    	PeerGroup mainChatGroup;    /* Il gruppo principale dell'applicazione */
        PeerGroupAdvertisement mCGAdv; /* Advertisement del mainChatGroup */
        
        System.out.println("Creazione dell'advertisement relativo al " + MAINGROUP_NAME);
        
        /*
         * E' necessario che l'ID del gruppo principale sia cablato nel codice, cosicché
         * il GID del gruppo principale sia univoco (condizione necessaria in fase di ricerca
         * del PeerGroupAdvertisement)
         */
        try {
        	/* Advertisement standard (servizi di base JXTA) */
            ModuleImplAdvertisement implAdv = netPG.getAllPurposePeerGroupImplAdvertisement();
            
            /* 
             * Il metodo PeerGroup.newGroup permette di creare e pubblicare in locale un
             * nuovo PeerGroup con l'invocazione di un singolo metodo
             */
            mainChatGroup = netPG.newGroup((PeerGroupID)(IDFactory.fromURI(new URI(MAINGROUP_ID))),
				  implAdv,
				  MAINGROUP_NAME,
				  "Gruppo principale per l'applicazione " + APP_NAME + " " + APP_VERSION);
            
            mCGAdv = mainChatGroup.getPeerGroupAdvertisement();
        }
        catch (Exception e) {
            System.out.println("Inizializzazione del gruppo principale per " +
            		APP_NAME + " " + APP_VERSION + " fallita");
            e.printStackTrace();
            return null; /* Fallita l'inizializzazione del gruppo principale */
        }
        /* Pubblicazione in remoto dell'advertisement per il mainChatGroup */
        System.out.println("Pubblicazione in remoto del " + MAINGROUP_NAME);
        try {
            discoverySv.remotePublish(mCGAdv);
            System.out.println("Pubblicazione riuscita");
        }
        catch (Exception e) {
            System.out.println("Errore nella pubblicazione dell'advertisement per il " +
            	MAINGROUP_NAME);
            e.printStackTrace();
            return null; /* Inizializzazione non completata */
        }
        
        return mainChatGroup;
    }
    
    /**
     * Metodo privato che attende fino alla connessione ad un RendezvousPeer o allo
     * scadere del timeout parametro imposta il peer locale come RDV
     * @param peerGroup NetPeerGroup
     * @param timeout Timeout in secondi
     * @throws Exception
     */
    private void waitRDVOrPromote (PeerGroup peerGroup, int timeout) throws Exception {
        RendezVousService rdv = peerGroup.getRendezVousService();
        if (rdv == null) {
            throw new RuntimeException("Nessun RendezvousService disponibile");
        }
        if(rdv.isRendezVous()) {
        	System.out.println("Il peer locale e' un RDV");
            return;
        }
        if (rdv.isConnectedToRendezVous() ) {
        	System.out.println("Il peer e' connesso a un RDV");
        	return;
        }
        rdv.setAutoStart(true, 34340);
        /* Attesa prima dell'auto-promozione a RDV *//*
        while (!rdv.isConnectedToRendezVous()&& !rdv.isRendezVous()){
            System.out.println("In attesa di un Rendezvous: " + timeout +
            		" secondi all'autopromozione a RDV");
            Thread.sleep(1*1000);
            if (--timeout <= 0) {	/* Tempo scaduto *//*
            	System.out.println("Il peer locale viene promosso RDV");
            	rdv.setAutoStart(true);
            	return;
            }
        }
        */
    }
    
    /**
     * Metodo privato che effettua l'accesso ad una chat
     * @param PeerGroupAdvertisement chatAdv
     * @return boolean Flag che indica la corretta (o errata) esecuzione della join
     */
    private boolean joinChat (PeerGroupAdvertisement chatAdv, String nickName) {
    	PeerGroup chat = null;
    	try {
    		chat = mainGroup.newGroup(chatAdv);
    	} catch (PeerGroupException pge) {
    		System.out.println("Fallita la creazione del gruppo a partire dall'advertisement");
    		pge.printStackTrace();
    		return false;
    	}
    	ChatInterface chatInterface = new ChatInterface(this, chat, nickName);
    	return true;
    }
    
    /**
     * Metodo per la creazione e pubblicazione di una chat pubblica
     * @param cName Nome della nuova chat
     * @param cDescr Stringa descrittiva della nuova chat
     * @return PeerGroupAdvertisement Advertisement della nuova chat, oppure null in caso di fallimento
     */
    public PeerGroupAdvertisement createStandardChat (String cName, String cDescr) {
    	
    	PeerGroupAdvertisement newChatAdvertisement;
    	try {
    		PeerGroup newChat = mainGroup.newGroup(null, impAdv, cName, cDescr);
    		newChatAdvertisement = newChat.getPeerGroupAdvertisement();
    		PeerGroupID GID = newChatAdvertisement.getPeerGroupID();
    		System.out.println("La stanza " + newChatAdvertisement.getName() + " con descrizione: " + newChatAdvertisement.getDescription() +
    				"\ne' stata creata" + "\n(GroupID: " + GID.toString() + ")");
    	} catch (PeerGroupException pge) {
    		System.out.println("Creazione della nuova chat fallita");
    		pge.printStackTrace();
    		return null;
    	}

        try {	 /* Pubblicazione in remoto dell'advertisement della chat */
            discoverySv.remotePublish(newChatAdvertisement);
            System.out.println("Pubblicazione avvenuta con successo");
        } catch (Exception e) {
            System.out.println("Pubblicazione fallita");
            e.printStackTrace();
            return null;
        }
        chatFinder.addChat(newChatAdvertisement);
        return newChatAdvertisement; /* La creazione e la pubblicazione sono andate a buon fine */
    }
}
