package net.jxta.chat.core;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;
import java.util.TimerTask;

import javax.swing.DefaultListModel;

import net.jxta.chat.presence.StatusService;
import net.jxta.chat.presence.UserStatusAdvertisement;
import net.jxta.chat.presence.UsersStatusListener;
import net.jxta.discovery.DiscoveryEvent;
import net.jxta.discovery.DiscoveryListener;
import net.jxta.discovery.DiscoveryService;
import net.jxta.document.Advertisement;
import net.jxta.protocol.DiscoveryResponseMsg;
import net.jxta.protocol.PeerAdvertisement;

/**
 * @author Matteo Tanca [tanca@cli.di.unipi.it]
 * @version 0.1
 * Classe che estende TimerTask ed implementa DiscoveryListener e UserStatusListener. Si occupa di ricercare
 * periodicamente nuovi peer all'interno di una chat, inserendoli in un'apposita tabella,
 * oltre a ricercare annunci di nuove pipe di propagazione, a cui collegarsi per ricevere
 * i messaggi da eventuali mittenti facenti capo a tali pipe
 */

public class PeersAndStatusFinder extends TimerTask implements DiscoveryListener,
															   UsersStatusListener {
	/*
	 * Tabella hash degli utenti in chat. Associa al PeerID un oggetto
	 * record che registra le informazioni sull'utente (nickname, stato) 
	 */
	private Hashtable<String, UserInfoRecord> peerTable;
	/*
	 * Insieme degli utenti andati offline di recente (serve per la
	 * cancellazione periodica dalla tabella e dalla lista di quegli
	 * utenti non piu' in chat)
	 */
	private Set<String> offlinePeers = null;
	
	private DiscoveryService discoverySv;
	private StatusService usersInfoSv;
	private ChatHandler handler;
	private DefaultListModel peerListModel = null;
	private String localInfo = null;
	
	public PeersAndStatusFinder (DiscoveryService d, StatusService s, ChatHandler ch, DefaultListModel pLM) {
		peerTable = new Hashtable<String, UserInfoRecord>();
		discoverySv = d;
        discoverySv.addDiscoveryListener(this);
        usersInfoSv = s;
        usersInfoSv.addListener(this);
        peerListModel = pLM;
        handler = ch;
        localInfo = buildPeerInfo(handler.getNickname(), handler.getPresenceIndex());
        peerTable.put(handler.getLocalPeerID(), new UserInfoRecord());
        peerListModel.addElement(localInfo);
        offlinePeers = new HashSet<String>();
	}
	
	/**
	 * Metodo invocato periodicamente (in quanto PeersAndPipesFinder estende TimerTask), che
	 * consente l'aggiornamento delle tabelle
	 */
	public void run() {
        try {
          /* Ripubblicazione periodica delle informazioni sul nickname e stato */
          usersInfoSv.publishUserInfo(handler.getPresenceIndex(), handler.getNickname());
          /* Cancellazione periodica degli utenti non in linea */
          Iterator<String> offlinePeersIterator = offlinePeers.iterator();
          while (offlinePeersIterator.hasNext()) {
        	  String peerID = offlinePeersIterator.next();
        	  UserInfoRecord rec = peerTable.remove(peerID);
        	  offlinePeers.remove(peerID);
        	  if (rec != null)
        		  peerListModel.removeElement(buildPeerInfo(rec.getNickname(), rec.getStatus()));
          }
          /* Ricerca di update dello stato degli utenti */
          refreshStatus();
        } catch(Exception e) {
        	System.out.println("Problemi nell'esecuzione del PeersAndPipesFinder (metodo run())");
            e.printStackTrace();
        }
    }
	
	/**
	 * Metodo che si occupa di inviare la richiesta di refresh dello stato degli utenti
	 * finora noti al servizio userInfoSv
	 *
	 */
	public void refreshStatus () {
		discoverySv.getRemoteAdvertisements(null, DiscoveryService.PEER, null, null, 100);
		Enumeration<String> keys = peerTable.keys();
		while (keys.hasMoreElements()) {
			String key = keys.nextElement();
			/* L'aggiornamento e' effettuato per tutti gli utenti a parte quello in locale */
			if (!key.equals(handler.getLocalPeerID()))
				usersInfoSv.findUserInfo(key);
		}
	}
	
	/**
	 * Metodo richiesto dall'interfaccia UsersStatusListener. Si occupa di verificare
	 * eventuali variazioni di stato | nickname dei peer indicizzati nella peerTable.
	 * In caso di aggiornamenti si occupa di riflettere tali cambiamenti nel modello
	 * della lista degli utenti
	 * @param userStatus UserStatusAdvertisement ricevuto dallo UsersStatusService
	 */
	public void userInfoEvent (UserStatusAdvertisement userStatus) {
		String key = userStatus.getPeerID().toString();
		
		/* Se e' presente in tabella si verificano i dati */ 
		if (peerTable.containsKey(key)) {
			UserInfoRecord userRec = peerTable.get(key);
			String actualNick = userStatus.getNickname();
			String oldNick = userRec.getNickname();
			int actualStatus = userStatus.getStatus();
			int oldStatus = userRec.getStatus();
			
			/* L'utente ha cambiato nick */
			if (!actualNick.equals(oldNick)) {
				userRec = new UserInfoRecord();
				userRec.setNickname(actualNick);
				userRec.setStatus(actualStatus);
				peerTable.put(key, userRec);
				String oldPeerInfo = buildPeerInfo(oldNick, oldStatus);
				String actualPeerInfo = buildPeerInfo(actualNick, actualStatus);
				boolean inList = refreshPeerListModel(oldPeerInfo, actualPeerInfo);
				handler.logNickChange(oldNick, actualNick);
				if (oldStatus != actualStatus && inList)	
					handler.logStatusChange(actualNick, actualStatus);
				return;
			}
			/* Stesso nick, ma stato differente */
			if (actualStatus != oldStatus) {
				userRec.setStatus(actualStatus);
				peerTable.put(key, userRec);
				String oldPeerInfo = buildPeerInfo(oldNick, oldStatus);
				String actualPeerInfo = buildPeerInfo(oldNick, actualStatus);
				boolean inList = refreshPeerListModel(oldPeerInfo, actualPeerInfo);
				if (inList)
					handler.logStatusChange(actualNick, actualStatus);
			}
			
			/* Da eliminare al prossimo refresh */
			if (actualStatus == StatusService.OFFLINE_STATUS)
				offlinePeers.add(key);
			
			/* Se la situazione non e' cambiata non si effettua alcun aggiornamento */
		}
	}
	
    /**
     * Implementazione del metodo discoveryEvent, richiesta per poter implementare l'interfaccia
     * DiscoveryListener, che permette di ricevere i risultati delle ricerche in maniera
     * asincrona
     * @param ev L'evento di Discovery
     */
    public void discoveryEvent (DiscoveryEvent ev) {

        DiscoveryResponseMsg res = ev.getResponse();

        Enumeration enumeration = res.getAdvertisements();
        
        /* Trovato qualcosa */
        if (enumeration != null ) {
        	while (enumeration.hasMoreElements()) {
        		Advertisement adv = (Advertisement)enumeration.nextElement();
        		String advType = adv.getAdvType();
        		/* Risposta di tipo PeerAdvertisement */
        		if (advType.equals(PeerAdvertisement.getAdvertisementType()))
        			addPeer((PeerAdvertisement)adv);
        		else if (advType.equals(UserStatusAdvertisement.getAdvertisementType()))
        			addPeer(((UserStatusAdvertisement)adv).getPeerID());
        	}
        }
    }
    
    /**
     * Metodo privato per l'inserimento di PeerID in tabella, con eventuale richiesta delle
     * informazioni sull'utente ad essi associato qualora si inseriscano nuovi elementi
	 * @param advertisement PeerAdvertisement da cui estrarre il PeerID
	 */
	private void addPeer (PeerAdvertisement advertisement) {
		String key = advertisement.getPeerID().toString();
		/*
		 * L'aggiunta deve essere effettuata solo se la tabella non contiene gia'
		 * un'associazione per il PeerID contenuto nel PeerAdvertisement esaminato
		 * Se non e' presente alcuna associazione si invia anche una richiesta di
		 * informazioni sul nick e stato del Peer appena scoperto
		 */
		if (!peerTable.containsKey(key)) {
			peerTable.put(key, new UserInfoRecord());
			usersInfoSv.findUserInfo(key);
		}
	}
	
	/**
     * Metodo privato per l'inserimento di PeerID in tabella, con eventuale richiesta delle
     * informazioni sull'utente ad essi associato qualora si inseriscano nuovi elementi
	 * @param advertisement PeerAdvertisement da cui estrarre il PeerID
	 */
	private void addPeer (String peerID) {
		/*
		 * L'aggiunta deve essere effettuata solo se la tabella non contiene gia'
		 * un'associazione per il PeerID contenuto nel PeerAdvertisement esaminato
		 * Se non e' presente alcuna associazione si invia anche una richiesta di
		 * informazioni sul nick e stato del Peer appena scoperto
		 */
		if (!peerTable.containsKey(peerID)) {
			peerTable.put(peerID, new UserInfoRecord());
			usersInfoSv.findUserInfo(peerID);
		}
	}
	
	/**
	 * Metodo per il refresh delle informazioni relative al peer locale
	 * (invocato dal chat handler alla modifica)
	 */
	public void refreshLocalInfo () {
		String newNick = handler.getNickname();
		int newStatus = handler.getPresenceIndex();
		usersInfoSv.publishUserInfo(newStatus, newNick);
		String newLocalInfo = buildPeerInfo(newNick, newStatus);
		refreshPeerListModel(localInfo, newLocalInfo);
		localInfo = newLocalInfo;
	}
    
    /**
     * Metodo privato per il refresh del modello usato per aggiornare la lista degli utenti
     * visualizzata dall'interfaccia
     * @return boolean Flag di effettiva presenza del vecchio contatto
     */
    private boolean refreshPeerListModel (String oldPeerInfo, String actualPeerInfo) {
    	boolean isIn = peerListModel.removeElement(oldPeerInfo);
    	peerListModel.addElement(actualPeerInfo);
    	return isIn;
    }
    
    /**
     * Metodo privato di supporto che gestisce la formattazione delle informazioni da
     * visualizzare nella lista utente
     * @param nick String
     * @param status Intero rappresentazione dello stato dell'utente
     * @return String La rappresentazione in formato String delle informazioni relative all'utente
     */
    private String buildPeerInfo (String nick, int status) {
    	String statusString = null;
    	switch (status) {
    		case StatusService.OFFLINE_STATUS:
    			statusString = "Non in linea";
    			break;
    		case StatusService.ONLINE_STATUS:
    			statusString = "In linea";
    			break;
    		case StatusService.AWAY_STATUS:
    			statusString = "Non al computer";
    			break;
    		case StatusService.BUSY_STATUS:
    			statusString = "Occupato";
    			break;
    		case StatusService.BACK_SOON_STATUS:
    			statusString = "Torno subito";
    	}
    	return nick + " (" + statusString + ")";
    }
}
