package net.jxta.chat.interfaces;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import javax.swing.JColorChooser;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JTextArea;

import net.jxta.chat.core.ChatHandler;
import net.jxta.chat.presence.StatusService;

/**
 * @author Matteo Tanca [tanca@cli.di.unipi.it]
 * @version 0.1
 * Listener degli eventi provenienti dall'interfaccia del gestore di chat
 * Si occupa di gestire click del mouse, eventi di chiusura, eventi di stato etc.
 */
public class ChatInterfaceEventListener implements MouseListener, ItemListener,
												   KeyListener, WindowListener {

	private ChatHandler handler;
	private ChatInterface chatInterface;
	private JTextArea inputArea;
	
	public ChatInterfaceEventListener (ChatInterface cI, ChatHandler cH, JTextArea iA) {
		chatInterface = cI;
		handler = cH;
		inputArea = iA;
	}
	
	public void mouseClicked(MouseEvent e) {
		/* Se non è stato premuto il bottone sinistro */
		if (e.getButton() != MouseEvent.BUTTON1)
			return;
		String eventGenerator = e.getComponent().getName();
		
		/* Click sul bottone 'Aggiorna stato utenti' */
		if (eventGenerator.equals("Refresh")) {
			handler.getPeersAndStatusFinder().refreshStatus();
		}
		/* Click sul bottone 'Invia messaggio' */
		else if (eventGenerator.equals("SendMsg")) {
			try {
				/* tentativo di invio di testo nullo...semplicemente la
				 * richiesta viene scartata
				 */
				if (inputArea.getText().length() == 0)
					return;
				String msg = inputArea.getText() + "\n";
				inputArea.setText(null);	/* elimina il testo digitato dalla finestra di input */
				handler.sendMsg(msg);
			} catch (NullPointerException npe) {
			}
		}
	}
	
	public void mousePressed(MouseEvent e) {
		/* Se non è stato premuto il bottone sinistro */
		if (e.getButton() != MouseEvent.BUTTON1)
			return;
		
		String eventGenerator = e.getComponent().getName();
		/* Click sul menu' 'Imposta nickname' */
		if (eventGenerator.equals("SetNickname")) {
			Object selection = JOptionPane.showInputDialog(chatInterface, null, "Impostazione nickname per la chat", JOptionPane.QUESTION_MESSAGE, null, null, handler.getNickname());
			if (selection == null)
				return;
			if (!selection.toString().equals(handler.getNickname()) && selection.toString().length() > 0)
				handler.setNickname((String)selection);
		}
		/* Click sul menu' 'Esci' */
		else if (eventGenerator.equals("Exit")) {
			handler.dispose();
		}
		/* Click sul menu' 'Salva cronologia' */
		else if (eventGenerator.equals("SaveHistory")) {
			JFileChooser chooser = new JFileChooser();
			if (chooser.showSaveDialog(chatInterface) != JFileChooser.APPROVE_OPTION)
				return;
			File f = chooser.getSelectedFile();
			if (f == null)
			   return;
			try {
				FileWriter out = new FileWriter(f);
				out.write(chatInterface.getHistory());
				out.close();
			} catch (IOException ioe) {
				 System.out.println("Errore nel salvataggio su " + f);
			}
		}
		/* Click sul menu' 'Colore dello sfondo e del testo' */
		else if (eventGenerator.equals("ChangeBackgroundItem")) {
			Color c = getNewColor();
			if (c != null)
				chatInterface.setTextBackground(c);
		}
		else if (eventGenerator.equals("ChangeTextColorItem")) {
			Color c = getNewColor();
			if (c != null)
				chatInterface.setTextColor(c);
		}

		/* Click sul menu' 'Font' */
		else if (eventGenerator.equals("ChangeTextItem")) {
			FontDialog dialog = new FontDialog(chatInterface);
			dialog.setVisible(true);
			if (dialog.isInputConfirmed()) {
				Font newFont = dialog.getCurrentFont();
				chatInterface.setCurrentFont(newFont);
			}
		}
		/* Click sul checkBox di log modifiche utenti */
		else if (eventGenerator.equals("LogCheckBox")) {
			handler.invertLogFlag();
		}
		/* Radio Button di stato */
		else {
			if (eventGenerator.equals("OnlineItem"))
				handler.setPresenceInfo(StatusService.ONLINE_STATUS);
			else if (eventGenerator.equals("AwayItem"))
				handler.setPresenceInfo(StatusService.AWAY_STATUS);
			else if (eventGenerator.equals("BackSoonItem"))
				handler.setPresenceInfo(StatusService.BACK_SOON_STATUS);
			else if (eventGenerator.equals("BusyItem"))
				handler.setPresenceInfo(StatusService.BUSY_STATUS);
			else if (eventGenerator.equals("OfflineItem"))
				handler.setPresenceInfo(StatusService.OFFLINE_STATUS);
		}
	}
	
	public void mouseReleased(MouseEvent e) {
		//non fa niente	
	}
	/**
	 * @param e KeyEvent Evento generato dalla pressione 
	 */
	public void keyReleased(KeyEvent e) {
		if (e.getKeyCode() == KeyEvent.VK_ENTER) {
			try {
				String msg = inputArea.getText();
				inputArea.setText(null);	/* elimina il testo digitato dalla finestra di input */
				/* 
				 * tentativo di invio di testo nullo (unico carattere
				 * \n)...semplicemente la richiesta viene scartata
				 */
				if (msg.length() <= 1)
					return;
				handler.sendMsg(msg);
			} catch (NullPointerException npe) {
			}
		}
	}
	
	/**
	 * Metodo privato che mostra un dialogo per la selezione del
	 * colore e lo restituisce qualora la selezione vada a buon
	 * fine, altrimenti restituisce null
	 * @return Color Il colore selezionato
	 */
	private Color getNewColor () {
		return JColorChooser.showDialog(chatInterface,
				  "Selezione colore", Color.white);
	}
	
	/**
	 * Implementazione del metodo richiesto dall'interfaccia ItemEventListener
	 * All'arrivo dell'evento il flag booleano dell'autoscroll viene invertito
	 * @param e ItemEvent Evento generato dalla selezione/deselezione della check box
	 */
	public void itemStateChanged(ItemEvent e) {
		chatInterface.invertHistoryScrollFlag();
	}

	/*
	 *****************************************************************************************
	 * METODI RICHIESTI DALLE INTERFACCE MOUSE/KEY LISTENER, MA INUTILIZZATI NELLO SPECIFICO *
	 *****************************************************************************************
	 */
	public void mouseEntered(MouseEvent e) {
		//non fa niente
	}
	public void mouseExited(MouseEvent e) {
		//non fa niente
	}
	
	public void keyPressed(KeyEvent e) {
		//non fa niente
	}
	public void keyTyped(KeyEvent e) {
		//non fa niente
	}

	/*
	 *****************************************************
	 * METODI RICHIESTI DALL' INTERFACCIA WINDOWLISTENER *
	 *****************************************************
	 */

	public void windowActivated(WindowEvent e) {
		
	}

	public void windowClosed(WindowEvent e) {
	}

	public void windowClosing(WindowEvent e) {
		handler.dispose();
	}

	public void windowDeactivated(WindowEvent e) {
		
	}

	public void windowDeiconified(WindowEvent e) {
		
	}

	public void windowIconified(WindowEvent e) {
		
	}

	public void windowOpened(WindowEvent e) {
		
	}	
}
