package net.jxta.chat.presence;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.UnknownServiceException;

import net.jxta.chat.core.PeerCore;
import net.jxta.document.AdvertisementFactory;
import net.jxta.id.IDFactory;
import net.jxta.platform.ModuleClassID;
import net.jxta.platform.ModuleSpecID;
import net.jxta.protocol.ModuleClassAdvertisement;
import net.jxta.protocol.ModuleImplAdvertisement;
import net.jxta.protocol.ModuleSpecAdvertisement;

/**
 * @author Matteo Tanca [tanca@cli.di.unipi.it]
 * @version 0.1
 * Classe di metodi statici di supporto per la creazione di Advertisement di tipo
 * MOduleClass, ModuleSpec, ModuleImpl
 */
public class AdvertisementCreationUtilities {
	
	/**
     * Metodo statico per la creazione di un ModuleClassAdvertisement a partire
     * dagli argomenti
     * @param moduleClassID ModuleClassID dell'Advertisement
     * @param advName Nome dell'Advertisement
     * @param advDescription Descrizione dell'Advertisement
     * @throws UnknownServiceException
     * @throws URISyntaxException
     * @return ModuleClassAdvertisement ModuleClassAdvertisement configurato secondo richiesta
     */
    public static ModuleClassAdvertisement createModuleClassAdvertisement (String moduleClassID,
    		                                 String advName, String advDescription) throws
    		                                 UnknownServiceException, URISyntaxException {
    	
        /* Conversione della stringa rappresentazione del ModuleClassID in in oggetto di tale tipo */
        ModuleClassID classID = (ModuleClassID)IDFactory.fromURI(new URI(moduleClassID));

        /*Creazione dell'advertisement ed impostazione dei parametri */
        ModuleClassAdvertisement advertisement = (ModuleClassAdvertisement)AdvertisementFactory.newAdvertisement(
                    							  ModuleClassAdvertisement.getAdvertisementType());

        advertisement.setModuleClassID(classID);
        advertisement.setName(advName);
        advertisement.setDescription(advDescription);

        return advertisement;
    }

    /**
     * Metodo statico per la creazione di un ModuleImplAdvertisement a partire dagli argomenti
     * forniti
     * @param peerGroupImplAdv ModuleImplAdvertisement del peerGroup genitore
     * @param moduleSpecAdv ModuleSpecAdvertisement
     * @param implDescription String descrizione dell'implementazione del modulo
     * @param code Codice identificativo dell'implementazione
     * @return ModuleImplAdvertisement ModuleImplAdvertisement configurato come da richiesta
     */
    public static ModuleImplAdvertisement createModuleImplAdvertisement (ModuleImplAdvertisement peerGroupImplAdv, 
    									    ModuleSpecAdvertisement moduleSpecAdv,
    									    String implDescription, String code) {
    	
        /* ModuleSpecID della specifica */
        ModuleSpecID specID = moduleSpecAdv.getModuleSpecID();

        /* Creazione dell'advertisement ed impostazione dei parametri */
        ModuleImplAdvertisement advertisement = (ModuleImplAdvertisement)AdvertisementFactory.newAdvertisement(
        										 ModuleImplAdvertisement.getAdvertisementType());
        advertisement.setModuleSpecID(specID);
        advertisement.setCode(code);
        advertisement.setCompat(peerGroupImplAdv.getCompat());
        advertisement.setDescription(implDescription);
        advertisement.setProvider("jxta_chat.com");
        advertisement.setUri("http://www.cli.di.unipi.it/~tanca/JXTA_chat.jar");

        return advertisement;
    }

    /**
     * Metodo statico per la creazione di un ModuleSpecAdvertisement con gli argomenti passati
     * @param moduleSpecID ModuleSpecID dell'advertisement
     * @param advName String Nome dell'advertisement
     * @param advDescription Stringa descrittiva dell'advertisement
     * @throws UnknownServiceException
     * @throws URISyntaxException
     * @return ModuleSpecAdvertisement L'advertisement che e' stato generato
     */
    public static ModuleSpecAdvertisement createModuleSpecAdvertisement (String moduleSpecID, String advName,
    					 					String advDescription)  throws UnknownServiceException,
    					 					URISyntaxException {
    	
        /* Crea l'oggetto ModuleSpecID a partire dalla stringa rappresentazione */
        ModuleSpecID specID = (ModuleSpecID) IDFactory.fromURI(new URI((moduleSpecID)));

        /* Creazione dell'advertisement ed impostazione dei parametri */
        ModuleSpecAdvertisement advertisement = (ModuleSpecAdvertisement)AdvertisementFactory.newAdvertisement(
                								 ModuleSpecAdvertisement.getAdvertisementType());
        advertisement.setCreator(PeerCore.APP_NAME + PeerCore.APP_VERSION);
        advertisement.setModuleSpecID(specID);
        advertisement.setDescription(advDescription);
        advertisement.setName(advName);
        advertisement.setSpecURI("http://www.cli.di.unipi.it/~tanca/JXTA_chat.jar");
        advertisement.setVersion(PeerCore.APP_VERSION);

        return advertisement;
    }
}
