package net.jxta.chat.presence;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;

import net.jxta.discovery.DiscoveryEvent;
import net.jxta.discovery.DiscoveryListener;
import net.jxta.discovery.DiscoveryService;
import net.jxta.document.Advertisement;
import net.jxta.document.AdvertisementFactory;
import net.jxta.document.MimeMediaType;
import net.jxta.exception.PeerGroupException;
import net.jxta.id.ID;
import net.jxta.peergroup.PeerGroup;
import net.jxta.platform.Module;
import net.jxta.protocol.DiscoveryResponseMsg;
import net.jxta.protocol.ModuleImplAdvertisement;
import net.jxta.service.Service;


/**
 * @author Matteo Tanca [tanca@cli.di.unipi.it]
 * @version 0.1
 * Implementazione del servizio informativo sullo stato degli utenti della chat, basata sul servizio DiscoveryService
 * fornito dal framework JXTA. Questo permette il caching delle informazioni sullo stato degli utenti
 */
public class StatusServiceImpl implements StatusService, DiscoveryListener {
    
    /* Reference module ID per l'implementazione dello UsersStatusService */
    public static final String refModuleSpecID = "urn:jxta:uuid-59A9A948905341119EAB8630EED42AB9F4611FF6377C4931AE71BE299B9F34DF06";
    /* Informazioni utili alla pubblicazione dell'advertisement */
    public static final String SPEC_DESCRIPTION = "Specifica dello UsersStatusService";

    /* 
     * Validita' temporale degli UserStatusAdvertisement in cache. Di default e' impostato a 5", in maniera
     * tale da limitare il fenomeno del caching di UserStatusAdvertisement inconsistenti
     */
    private static final long DEFAULT_EXPIRATION = 1000 * 7;  //7 s
    
    /* Validita' temporale degli UserStatusAdvertisement. Di base la scadenza e' fissata a 15" */
    private static final long DEFAULT_LIFETIME = 1000 * 12; //12 s
    
    /*
     * Variabili necessarie per il corretto funzionamento del servizio: PeerGroup che lo fornisce, DiscoveryService
     * utilizzato, implementazione dell'Advertisement, peerID del peer locale (necessario per la pubblicazione dello
     * stato) e insieme dei listener registrati (destinatari degli eventi generati dal servizio)
     */
    private DiscoveryService discoverySv = null;
    private Advertisement statusServiceModuleImplAdvertisement = null;
    private String localPeerID = null;
    private PeerGroup chat = null;
    private Vector<UsersStatusListener> usersStatusListeners = null;


    public StatusServiceImpl () {
        super();
        usersStatusListeners = new Vector<UsersStatusListener>();
    }

    /**
     * Metodo sincronizzato per la registrazione di un listener. In seguito all'inserimento ogni
     * evento sara' propagato anche ad esso, fino a un'eventuale rimozione.
     * La sincronizzazione evita inconsistenze sul vettore dei listener
     * @param usl UsersStatusListener da registrare
     */
    public synchronized void addListener (UsersStatusListener usl) {
        usersStatusListeners.add(usl);
    }
    
    /**
     * Metodo sincronizzato per la rimozione di un listener. Restituisce il risultato della
     * operazione
     * @return boolean true se l'elemento e' stato rimosso, false altrimenti
     */
	public synchronized boolean removeListener (UsersStatusListener usl) {
		return usersStatusListeners.remove(usl);
	}

    /**
     * Implementazione richiesta dall'interfaccia DiscoveryListener.
     * E' necessario filtrare i risultati delle richieste di discovery, che non saranno in
     * generale relativi ai soli UserStatusAdvertisement.
     * Gli eventi che superano il filtro sono propagati ai listener registrati
     * @param event DiscoveryEvent ricevuto dal servizio di discovery su cui this e' registrato come DiscoveryListener
     */
    public void discoveryEvent(DiscoveryEvent event) {
        
    	DiscoveryResponseMsg response = event.getResponse();
        Enumeration responses = response.getResponses();
        
        /*
         * Per ogni risposta trovata verifica che si tratti di uno UserStatusAdvertisement
         * e nel caso lo propaga ai listener, altrimenti la risposta e' tralasciata
         */
        while (responses.hasMoreElements())
        {
            String elem = (String)responses.nextElement();
            
            /* 
             * Se l'elemento non è nullo si tenta di processarlo come
             * UserStatusAdvertisement.
             */
            if (elem != null) {
            	
                try {
                	/* Costruisce il byteStream relativo all'oggetto elem */
                    ByteArrayInputStream stream = new ByteArrayInputStream(elem.getBytes());
                    
                    Advertisement advertisement = AdvertisementFactory.newAdvertisement(
                            					  new MimeMediaType("text/xml"), stream);
                    
                    if (advertisement.getAdvType().equals(UserStatusAdvertisement.getAdvertisementType()))
                    	/* 
                    	 * Se si arriva fin qui si tratta davvero di uno UserStatusAdvertisement
                    	 * e lo si invia a tutti i listener registrati nel vettore
                    	 */
                    	for (int i = 0; i < usersStatusListeners.size(); i++)
                    		usersStatusListeners.get(i).userInfoEvent((UserStatusAdvertisement)advertisement);
                }
                catch (IOException ioe) {
                    System.out.println("Errore nel processing delle risposte nel" +
                    		" DiscoveryEvent dello UserStatusService");
                    ioe.printStackTrace();
                }
            }
        }
    }
    

    /**
     * Metodo che consente all'utente di pubblicare il proprio nome e stato utente nella chat
     * @param userStatus Intero rappresentazione dello stato utente
     * @param nickname Nickname che l'utente desidera utilizzare nella chat a cui fa capo lo UsersStatusService
     */
    public void publishUserInfo (int status, String nickname) {
    	/* Creazione dello UserStatusAdvertisement mediante chiamata all'AdvertisementFactory */
    	UserStatusAdvertisement advertisement = (UserStatusAdvertisement)AdvertisementFactory.newAdvertisement(
    											 UserStatusAdvertisement.getAdvertisementType());

    	/* Inserimento delle informazioni da inviare */
    	advertisement.setStatus(status);
    	advertisement.setNickname(nickname);
    	advertisement.setPeerID(localPeerID);
    	/* Pubblicazione dello UserStatusAdvertisement in locale, seguita dall pubblicazione in remoto */
    	try {
    		discoverySv.publish(advertisement, DEFAULT_LIFETIME, DEFAULT_EXPIRATION);
    	} catch (IOException ioe) {
    		System.out.println("Problema nella pubblicazione locale dello UserStatusAdvertisement");
    		ioe.printStackTrace();
    	}

    	discoverySv.remotePublish(advertisement, DEFAULT_EXPIRATION);
    }

    /**
     * Metodo di ricerca di UserStatusAdvertisement sulla base del peerID
     * @param peerID Stringa rappresentazione del peerID dell'utente di cui si vuole conoscere
     * 	      lo stato
     */
	public void findUserInfo (String peerID) {
		
		/*
		 * Per prima cosa effettua una ricerca nella cache locale, notificando eventuali
		 * risultati ai listener registrati
		 */
		try	{
			Enumeration en = discoverySv.getLocalAdvertisements(
					DiscoveryService.ADV, UserStatusAdvertisement.PEER_ID_TAG, peerID);
			while (en.hasMoreElements()) {
				UserStatusAdvertisement advertisement = (UserStatusAdvertisement) en.nextElement();
				for (int i = 0; i < usersStatusListeners.size(); i++)
                	usersStatusListeners.get(i).userInfoEvent(advertisement);
			}
		} catch (IOException ioe) {
			System.out.println("Errore nella ricerca di UserStatusAdvertisement in locale");
			ioe.printStackTrace();
        }
		
		/* 
		 * Invio in remoto di una richiesta limitata ad Advertisement aventi un campo
		 * PEER_ID_TAG di valore peerID, con limite  di risposte pari a 6
		 */
		discoverySv.getRemoteAdvertisements(null, DiscoveryService.ADV, UserStatusAdvertisement.PEER_ID_TAG,
											peerID, 1, null);
	}

	/**
	 * Metodo richiesto dall'interfaccia Service
	 * @return Advertisement L'implementazione dell'advertisement
	 */
	public Advertisement getImplAdvertisement() {
		return statusServiceModuleImplAdvertisement;
	}

	/**
	 * Metodo richiesto dall'interfaccia Service
	 * @return Service L'oggetto che implementa l'interfaccia
	 */
	public Service getInterface() {
		return this;
	}

	/**
	 * Metodo che mette in esecuzione il servizio, richiesto dall' interfaccia
	 * Service
	 * @param args Array di parametri da fornire in fase di avvio
	 * @return int Restituisce 0 per indicare che il servizio e' partito
	 */
	public int startApp (String[] args) {
		/* 
         * Permette di utilizzare l'AdvertisementFactory per istanziare gli
         * UserStatusAdvertisement, come viene raccomandato dalle linee guida JXTA
         */
        AdvertisementFactory.registerAdvertisementInstance(UserStatusAdvertisement.getAdvertisementType(),
                			 new UserStatusAdvertisement.Instantiator());
    
        /* Nel caso l'inizializzazione vada per le lunghe */
        if (chat.getRendezVousService() == null)
        	return Module.START_AGAIN_PROGRESS;
        if (chat.getDiscoveryService() == null)
        	return Module.START_AGAIN_PROGRESS;
        
		discoverySv = chat.getDiscoveryService();
		/* Si registra direttamente come listener, invece di registrarsi ad ogni richiesta */
		discoverySv.addDiscoveryListener(this);
		
		return Module.START_OK;
	}

	/**
	 * Metodo che ferma il servizio, richiesto dall'interfaccia Service
	 */
	public void stopApp() {
		discoverySv.removeDiscoveryListener(this);
		discoverySv = null;
		usersStatusListeners.removeAllElements();
	}
	
	/**
     * Metodo per l'inizializzazione del servizio informativo, richiesto
     * dall'interfaccia Service. Si noti che lo UsersStatusAdvertisement
     * DEVE essere inizializzato prima dell'invocazione di startApp(), pena il non funzionamento
     * per non inizializzazione delle variabili
     * @param chat Il PeerGroup associato al servizio
     * @param serviceID L'identificatore assegnato al servizio
     * @param implAdv ModuleImplAdvertisement del servizio
     * @throws PeerGroupException Richiesta dall'interfaccia, ma non lanciata da questa
     * 		   particolare implementazione
     */
    public void init(PeerGroup chat, ID serviceID, Advertisement implAdv) throws PeerGroupException {
        
    	/* Inizializza la ModuleImplAdvertisement */
        statusServiceModuleImplAdvertisement = (ModuleImplAdvertisement)implAdv;

        /* Associazione della chat su cui opera il servizio */
        this.chat = chat;

        // Inizializza il peerID locale */
        localPeerID = chat.getPeerID().toString();      
    }
}
