package net.jxta.chat.presence;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.util.Enumeration;

import net.jxta.document.Advertisement;
import net.jxta.document.AdvertisementFactory;
import net.jxta.document.Document;
import net.jxta.document.Element;
import net.jxta.document.MimeMediaType;
import net.jxta.document.StructuredDocument;
import net.jxta.document.StructuredDocumentFactory;
import net.jxta.document.StructuredTextDocument;
import net.jxta.document.TextElement;
import net.jxta.id.ID;

/**
 * @author Matteo Tanca [tanca@cli.di.unipi.it]
 * @version 0.1
 * Estensione (astratta) del tipo Advertisement. Definisce un nuovo tipo di advertisement,
 * che permette di pubblicare informazioni sul generico utente di una chat e di condividerle
 * con gli altri utenti
 */
public class UserStatusAdvertisement extends Advertisement {
    /* Stringa identificativa del tipo dell'Advertisement */
    private static final String ADVERTISEMENT_TYPE = "UserStatusAdvertisement";
    
    /* 
     * Stringa identificativa del tipo di codifica utilizzata per la trasmissione dello
     * UserStatusAdvertisement
     */
    private static final String MIME_TYPE = "text/xml";
    
    
    /* Stringhe identificative delle componenti dello UserStatusAdvertisement */
    private static final String NICKNAME_TAG = "Nickname";
    public static final String PEER_ID_TAG = "PeerID";
    private static final String STATUS_TAG = "UserStatus";
    
    /* Nickname dell'utente */
    private String nickname = null;
    /* peerID corrispondente al Peer su cui risiede l'utente */
    private String peerID = null;
    /* 
     * Intero rappresentante lo stato dell'utente (le costanti associate sono definite nello
     * UsersStatusService
     */
    private int status = StatusService.ONLINE_STATUS;

    /**
     * Costruttore di default
     * @return UserStatusAdvertisement
     */
    public UserStatusAdvertisement () {
        super();
    }

    /**
     * Costruttore che restituisce un'istanza di UserStatusAdvertisement a partire dallo
     * stream di input parametro
     * @param inputStream L'input sorgente
     * @throws IOException Qualora non sia generabile lo UserStatusAdvertisement partendo dallo
     * 	       stream
     */
    public UserStatusAdvertisement (InputStream inputStream) throws IOException {
        super();
        /* Documento XML strutturato */
        StructuredTextDocument document = (StructuredTextDocument)
        								  StructuredDocumentFactory.newStructuredDocument(
        							      new MimeMediaType(MIME_TYPE), inputStream);

        parseAdvertisement(document);
    }

    /**
     * Costruttore che restituisce un'istanza di UserStatusAdvertisement a partire dal
     * documento fornito come argomento
     * @param document La sorgente dei dati da inserire nell'advertisement
     * @throws IllegalArgumentException Qualora document non sia un TextElement
     */
    public UserStatusAdvertisement (Element document) throws IllegalArgumentException {
        super();
        parseAdvertisement((TextElement) document);
    }
    
    /**
     * Metodo per impostare il nickname
     * @param String Il nickname dell'utente
     */
    public void setNickname(String nick) {
        nickname = nick;
    }
    
    /**
     * Metodo che restituisce la stringa del nickname
     * @return String Il nickname dell'utente
     */
    public String getNickname() {
        return nickname;
    }

    /**
     * Metodo per impostare il peerID associato all'utente
     * @param String peerID associato all'utente
     */
    public void setPeerID(String peerID) {
        this.peerID = peerID;
    }
    
    /**
     * Metodo che restituisce la stringa rappresentante il peerID associato all'utente
     * @return String La rappresentazione del peerID associato all'utente
     */
    public String getPeerID() {
        return peerID;
    }

    /**
     * Metodo per impostare lo stato dell'utente
     * @param int Intero rappresentazione dello stato
     */
    public void setStatus(int userStatus) {
        status = userStatus;
    }
    
    /**
     * Metodo che restituisce l'intero rappresentazione dello stato
     * @return int Intero rappresentazione dello stato
     */
    public int getStatus() {
        return status;
    }
    
   
    /**********************************************************
     * IMPLEMENTAZIONE DEI METODI RICHIESTI DALLA SUPERCLASSE *
     **********************************************************/
    
    /**
     * Metodo che restituisce il tipo dell'Advertisement
     * @return String La stringa indicatrice del tipo UserStatusAdvertisement
     */
    public static String getAdvertisementType()
    {
        return ADVERTISEMENT_TYPE;
    }

    /**
     * Metodo che restituisce un identificatore unico per il tipo UserStatusAdvertisement
     * Sfrutta il fatto che restituendo ID.nullID il caching del documento viene comunque
     * effettuato, utilizzandone l'hash.
     * @return ID L'ID nullo, come definito nella classe ID
     */
    public ID getID() {
        return ID.nullID;
    }

    /**
     * Metodo che restituisce un Document rappresentazione dell'albero del documento contenuto
     * nello UserStatusAdvertisement
     * @param Il tipo di codifica utilizzato
     * @return Document
     * @throws IllegalArgumentException Qualora il peerID o il nickname siano nulli
     */
    public Document getDocument(MimeMediaType mimeType) throws IllegalArgumentException {
        if ((getPeerID() == null) || (getNickname() == null))
        	throw new IllegalArgumentException ("Argomenti insufficienti");

        StructuredDocument document = (StructuredTextDocument)
        	StructuredDocumentFactory.newStructuredDocument(mimeType, getAdvertisementType());
        
        /* Costruisce il documento coi dati reperiti da this */
        Element element;
          
        element = document.createElement(PEER_ID_TAG, getPeerID());
        document.appendChild(element);

        element = document.createElement(NICKNAME_TAG, getNickname());
        document.appendChild(element);

        /* Se lo stato non e' presente, sara' impostato al default, cioe' 'Online' (1) */
        element = document.createElement(STATUS_TAG, Integer.toString(getStatus()));
        document.appendChild(element);

        /* Restituisce il documento costruito */
        return document;
    }

    /**
     * Restituisce l'insieme di campi che possono essere utilizzati come indici
     * @return String[] L'array di campi da utilizzare per l'indicizzazione dello UserStatusAdvertisement
     * 	       quando viene posto in cache
     */
    public String[] getIndexFields() {
        return new String[] {PEER_ID_TAG, NICKNAME_TAG, STATUS_TAG};
    }
           
    /**
     * Metodo che effettua il parsing dello UserStatusAdvertisement a partire dal documento
     * parametro
     * @param TextElement L'oggetto che si suppone contenga i dati relativi allo UserStatusAdvertisement
     * @throws IllegalArgumentException Qualora l'argomento non rappresenti uno UserStatusAdvertisement
     */
    public void parseAdvertisement(TextElement document) throws IllegalArgumentException {
        if (!document.getName().equals(getAdvertisementType()))
        	throw new IllegalArgumentException("L'argomento non rappresenta uno UserStatusAdvertisement");
        
        /* Enumerazione degli elementi dell'albero rappresentato da document */
        Enumeration elements = document.getChildren();

        while (elements.hasMoreElements()) {
        	TextElement element = (TextElement) elements.nextElement();

        	if (element.getName().equals(NICKNAME_TAG))
        		setNickname(element.getTextValue());
        	else if (element.getName().equals(STATUS_TAG))
        		setStatus(Integer.parseInt(element.getTextValue()));
        	else if (element.getName().equals(PEER_ID_TAG))
        		setPeerID(element.getTextValue());
        }
    }

    /**
     * Metodo toString per l'oggetto UserStatusAdvertisement
     * @return String La stringa di rappresentazione XML di this
     */
    public String toString() {
        try {
            StringWriter out = new StringWriter();
            StructuredTextDocument doc = (StructuredTextDocument)getDocument(new MimeMediaType(MIME_TYPE));
            doc.sendToWriter(out);
            return out.toString();
        }
        catch (Exception e) {
            return "";
        }
    }
    
    /**
     * Classe di supporto che implementa l'istanziatore per la classe UserStatusAdvertisement
     */
    public static class Instantiator implements AdvertisementFactory.Instantiator {
    	
    	/**
    	 * Metodo che restituisce il tipo identificativo dell'advertisement
    	 * @return String Il tipo dell'advertisement
    	 */
    	public String getAdvertisementType() {
    		return UserStatusAdvertisement.getAdvertisementType();
    	}

    	/**
    	 * Restituisce una nuova istanza di UserStatusAdvertisement
    	 * @return Advertisement UserStatusAdvertisement
    	 */
    	public Advertisement newInstance() {
    		return new UserStatusAdvertisement();
    	}

    	/**
    	 * Restituisce una nuova istanza di UserStatusAdvertisement con radice l'elemento
    	 * passato come argomento
    	 * @param   root Element
    	 * @return  Advertisement UserStatusAdvertisement popolato con l'elemento radice
    	 */
    	public Advertisement newInstance(Element root) {
    		return new UserStatusAdvertisement(root);
    	}
    };
}
